@@ -4,11 +4,12 @@ Released into the public domain by Legoktm, 2013
 """
 
 import hashlib
-import memcache
 import mwparserfromhell
 import pywikibot
 from pywikibot.data import api
 import re
+import redis
+import sys
 
 
 #normalize/shorten a few names
@@ -17,7 +18,16 @@ norm = {'target required claim': 'target',
         'single value': 'single',
         'unique value': 'unique',
         }
-mc = memcache.Client(['tools-mc'])  # TODO: Make this configurable
+
+
+def get_mc_serv():
+    serv = 'tools-redis'
+    for arg in sys.argv:
+        if arg.startswith('--memcache'):
+            serv = arg[11:]
+    return serv
+
+mc = redis.StrictRedis(get_mc_serv())  # TODO: Make this configurable
 expiry = 60 * 60 * 24  # One day
 
 
@@ -29,10 +39,15 @@ def normalize(name):
 class WDProperty(pywikibot.PropertyPage):
     def md5(self):
         # Hopefully this is unique enough.
-        return hashlib.md5('wdapi' + self.getID()).hexdigest()
+        return hashlib.md5('wdapi' + self.getID() + str(self.repo)).hexdigest()
 
-    def get(self, force=False, *args):
-        return_this = super(pywikibot.PropertyPage, self).get(force, *args)  # Do it cuz
+    def get(self, force=False, fetch_text=True, cache=True, *args):
+        # Realistically no one even wants the property info, and datatype is its own function.
+        # Cache controls only saving as cache, not fetching from it
+        if fetch_text:
+            return_this = super(pywikibot.PropertyPage, self).get(force, *args)  # Do it cuz
+        else:
+            return_this = {}
         # Check that we don't already have it stored
         if not force and hasattr(self, '_constraints'):
             return return_this
@@ -66,7 +81,7 @@ class WDProperty(pywikibot.PropertyPage):
                 elif nm == 'oneof':
                     values = unicode(temp.get('values').value)
                     values = pywikibot.removeDisabledParts(values, tags=['comments'])
-                    values = values.replace('{{Q|', '').replace('}}', '')
+                    values = values.replace('{{Q|', '').replace('{{q|', '').replace('}}', '')
                     values = values.split(', ')
                     d[nm] = list()
                     for v in values:
@@ -79,18 +94,19 @@ class WDProperty(pywikibot.PropertyPage):
                     d[nm] = ''  # Just set a key like the API does
 
         self._constraints = d
-        mc.set(self.md5(), self._constraints, expiry)
+        if cache:
+            mc.set(self.md5(), self._constraints, expiry)
         return return_this
 
     def constraints(self, force=False):
         if force or not hasattr(self, '_constraints'):
-            self.get(force=force)
+            self.get(force=force, fetch_text=False)
         return self._constraints
 
 
 def canClaimBeAdded(item, claim, checkDupe=True):
     prop = WDProperty(item.repo, claim.getID())
-    prop.get()
+    prop.get(fetch_text=False)
     if not hasattr(item, '_content'):
         # TODO: Not all constraints require fetching this, so it should be lazy
         item.get()
@@ -117,8 +133,8 @@ def canClaimBeAdded(item, claim, checkDupe=True):
     return True, None
 
 
-def createItem(page):
-    summary = 'Importing from [[:w:{0}:{1}]]'.format(page.site.language(), page.title())
+def createItem(page, dontactuallysave=False):
+    summary = u'Importing from [[:w:{0}:{1}]]'.format(page.site.language(), page.title())
     gen = api.PropertyGenerator('langlinks', titles=page.title(), lllimit='max',
                                 site=page.site,
                                 )
@@ -127,7 +143,7 @@ def createItem(page):
     for c in gen:
         if 'langlinks' in c:
             for b in c['langlinks']:
-                link = {'site': b['lang'].replace('-', '_') + 'wiki',
+                link = {'site': page.site.dbName(),
                         'title': b['*'],
                         }
                 label = {'language': b['lang'],
@@ -145,6 +161,8 @@ def createItem(page):
     data = {'sitelinks': sitelinks,
             'labels': labels,
             }
+    if dontactuallysave:
+        return data
     repo = page.site.data_repository()
     result = repo.editEntity({}, data, bot=True, summary=summary)
     if 'success' in result and result['success'] == 1:
